import {inject, customElement} from 'aurelia-framework';
import {Router} from 'aurelia-router';
import {KaApi} from 'ka-components';

import navigation from 'navigation';

@customElement('ka-layout')
@inject(Element, Router, KaApi)
export class KaLayout {

  // Internal bindings
  _collapsed = false;
  _navigation = navigation;
  _intervals = [];

  constructor(element, router, api) {
    this.element = element;
    this.router = router;
    this.api = api;
    document.addEventListener('click', e => {
      if (e.target.closest('.no-toggle')) return;
      if (e.target.classList.contains('expand') || e.target.classList.contains('collapse')) return;
      let isBurger = e.target === this.burgerAside || e.target === this.burgerHeader || (e.target.parentElement && (e.target.parentElement === this.burgerAside || e.target.parentElement === this.burgerHeader));
      if (!isBurger && !this._collapsed && window.innerWidth < 860) this.toggle();
    });
  }

  attached() {
    if (window.innerWidth < 860) this.toggle();
    this.activateBadges(this._navigation);
  }
  detached() {
    for (let interval of this._intervals) clearInterval(interval);
  }

  toggle() {
    this._collapsed = !this._collapsed;
    if (this._collapsed) this.element.classList.add('collapsed'); else this.element.classList.remove('collapsed');
  }

  activateBadges(items) {
    for (let item of items) {
      if (item.nav && item.nav.length > 0 && this.authorized(item)) this.activateBadges(item.nav);
      if (!item.badge || !item.badge.promise || !this.authorized(item)) continue;
      let callback = () => {
        new Promise((resolve, reject) => { return item.badge.promise(resolve, reject, this.api) }).then(x => {
          item.badge.value = x;
        }).catch(() => { item.badge.value = null; });
      };
      this._intervals.push(setInterval(callback, item.badge.interval || 5000));
      callback();
    }
  }

  authorized(item) {
    let userGroups = this.api.user.group || this.api.user.groups || [];
    if (!Array.isArray(userGroups)) userGroups = [userGroups];
    return !item.authGroups || (item.authGroups && userGroups.some(e => item.authGroups.includes(e)));
  }

  expand(item) {
    item.collapsed = false;
  }
  collapse(item) {
    item.collapsed = true;
  }
}
