import {inject, customElement, observable} from 'aurelia-framework';
import {Router} from 'aurelia-router';
import {KaApi} from '../ka-api/ka-api';
import environment from 'environment';

@customElement('ka-navbar')
@inject(Element, Router, KaApi)
export class KaNavbar {
  @observable() visible = false;

  appName = environment.appName;

  constructor(element, router, api) {
    this.element = element;
    this.router = router;
    this.api = api;
    document.addEventListener('click', e => {
      if (!this.visible) return;
      let target = e.target;
      while (target) {
        if (target === this.nav) return;
        if (target === this.button) return;
        target = target.parentNode;
      }
      this.hide();
    });
  }

  attached() {
    this.routes = [];
    for (let route of this.router.navigation) {
      if (!this.api.user && !route.settings.authGroups)
        this.routes.push(route);
      else if (this.api.user && !route.settings.authGroups)
        this.routes.push(route);
      else if (this.api.user && route.settings.authGroups && this.api.user.groups && this.api.user.groups.some(e => route.settings.authGroups.includes(e)))
        this.routes.push(route);
    }
  }

  hide() {
    this.visible = false;
  }

  show() {
    this.visible = true;
  }

  toggle() {
    this.visible = !this.visible;
  }

}
