import {inject, customElement, bindable, bindingMode, InlineViewStrategy} from 'aurelia-framework';
import {Router} from 'aurelia-router';
import {KaDialog} from '../ka-dialog/ka-dialog';
import {KaToast} from '../ka-toast/ka-toast';
import moment from 'moment';

@customElement('ka-table')
@inject(Element, Router, KaDialog, KaToast)
export class KaTable {

  @bindable() records = null;
  @bindable() schema = null;
  @bindable() export_schema = null;
  @bindable() columns_schema = null;
  @bindable() filters = null;
  @bindable() endpoint = null;
  @bindable() limit = null;
  @bindable() depth = null;
  @bindable() client = null; // Aurelia's http client to be used for fetching data
  @bindable() columns = null; // Kamaji's fields to use as table columns, defaults to all fields
  @bindable() filtrables = null; // Kamaji's fields to use as filters, defaults to all fields
  @bindable() sortables = null; // Kamaji's fields to set as sortable if in table columns
  @bindable() editables = null; // Kamaji's fields to set as editable
  @bindable() exportables = null; // Kamaji's fields to set as available for export
  @bindable() prefilter = null; // Predefined hardcoded filter string
  @bindable() routes = { add: null, edit: null, dialog: false, data: {} }; // Routes names for actions - if dialog is true, then open in dialog
  @bindable() actions = ["filter","export","add","edit"];
  @bindable() params = {};
  @bindable() query = '';

  schema_loaded = () => {};
  filters_loaded = () => {};
  parse_query = (query) => { return query; };
  records_loaded = () => {};

  id = null;
  settings = {
    filters: null,
    exports: null,
    sort: {},
    sort_stack: [],
    sort_columns: {},
    offset: 0,
    limit: '10'
  }
  config_operators = {
    text: {
      field: 'operator',
      control: 'combo',
      datapreload: true,
      datasource: [
        {value: '=', text: 'Uguale a'},
        {value: '!=', text: 'Diverso da'},
        {value: '^=', text: 'Inizia con'},
        {value: '~', text: 'Contiene'},
        {value: '!~', text: 'Non contiene'}
      ]
    },
    json: {
      field: 'operator',
      control: 'combo',
      datapreload: true,
      datasource: [
        {value: '~', text: 'Contiene'}
      ]
    },
    number: {
      field: 'operator',
      control: 'combo',
      datapreload: true,
      datasource: [
        {value: '=', text: 'Uguale a'},
        {value: '!=', text: 'Diverso da'},
        {value: '>=', text: 'Maggiore uguale a'},
        {value: '<=', text: 'Minore uguale a'}
      ]
    }
  }
  config_metas = [
    { field: 'id', label: 'Id', control: 'number' },
    { field: 'date_created', label: 'Data creazione', control: 'date'},
    { field: 'user_created', label: 'Creato da', control: 'number'},
    { field: 'date_modified', label: 'Data ultima modifica', control: 'date'},
    { field: 'user_modified', label: 'Modificato da', control: 'number'}
  ]
  config_options = {
    limit: { field:'option-limit', label: 'Record per pagina', control: 'combo', datapreload: true, datasource: [{value:'10', text:'10'}, {value:'25', text:'25'}, {value:'50', text:'50'}, {value:'100', text:'100'}, {value:'200', text:'200'}] },
    pages: { field:'option-pages', label: 'Salta alla pagina', control: 'combo', datapreload: true, datasource: [] }
  }
  position = null;
  hasFilters = false;
  isLoading = true;
  schemaLoaded = false;
  error = null;

  constructor (element, router, dialog, toast) {
    this.element = element;
    this.router = router;
    this.dialog = dialog;
    this.toast = toast;
  }

  bind(bindingContext, overrideContext) {
    this.parent = bindingContext;
  }

  detached() {
    this.hide_options();
    this.hide_filters();
    window.removeEventListener('resize', window[`ka-table-resize-handler-${this.id}`]);
    delete(window[`ka-table-resize-handler-${this.id}`]);
  }

  attached () {
    if (!this.endpoint && this.parent.endpoint) this.endpoint = this.parent.endpoint;
    if (!this.client && this.parent.api && this.parent.api.client) this.client = this.parent.api.client;
    if (this.columns && typeof this.columns == 'string') this.columns = this.columns.split(',');
    if (this.filtrables && typeof this.filtrables == 'string') this.filtrables = this.filtrables.split(',');
    if (this.sortables && typeof this.sortables === 'string') this.sortables = this.sortables.split(',');
    if (this.exportables && typeof this.exportables == 'string') this.exportables = this.exportables.split(',');
    if (this.actions && typeof this.actions == 'string') this.actions = this.actions === "" ? [] : this.actions.split(',');
    if (this.limit) this.settings.limit = String(this.limit);

    let userId = this.parent.api && this.parent.api.user && this.parent.api.user.id ? this.parent.api.user.id : null;
    let appVer = this.parent.api && this.parent.api.environment && this.parent.api.environment.version ? this.parent.api.environment.version : null;
    appVer = !appVer && this.parent.environment && this.parent.environment.version ? this.parent.environment.version : appVer;

    this.id = btoa('§'+userId+'-'+appVer+'§'+this.prefilter+'§'+this.endpoint+'§'+location.pathname+'§'+(this.columns?this.columns.join(''):'')).replace(/[\W_]+/g,'').slice(0, 120);
    this.element.setAttribute('id', this.id);
    /* Overwrite default settings with local storage if any */
    if (localStorage.getItem(this.id)) {
      this.settings = JSON.parse(localStorage.getItem(this.id));
    } else if (sessionStorage.getItem(this.id)) {
      this.settings = JSON.parse(sessionStorage.getItem(this.id));
    }
    this.init();

    this.element.querySelector('.ka-wrapper').addEventListener('scroll', event => {
      event.target.querySelectorAll('.floater').forEach(floater => {
        floater.style.transform = `translateX(${event.target.scrollLeft}px)`;
      });
    });

    // Create evaled resize linstener function
    eval(`window['ka-table-resize-handler-${this.id}'] = function() {
      var element = document.querySelector('#${this.id}');
      if (!element) return;
      if (element.classList.contains('full-height')) {
        element.style.height = parseInt(window.innerHeight - element.getBoundingClientRect().top) + 'px';
      }
      var table = element.querySelector('table');
      table.querySelectorAll('tbody td.ka-tr-tools:last-child').forEach(function (element) {
        var floater = element.querySelector('.floater');
        if (!floater) return;
        var h = element.getBoundingClientRect().height;
        var w = floater.getBoundingClientRect().width;
        if (floater && h && w) {
          floater.classList.add('float');
          floater.style.height = h + 'px';
          element.style.minWidth = w + 'px';
        }
      });
    }`);
    window.addEventListener('resize', window[`ka-table-resize-handler-${this.id}`]);
  }

  init() {
    if (!this.client || !this.endpoint)  return console.warn('[mk-table] missing client or endpoint configuration');
    this.isLoading = true;
    this.error = null;
    this.hide_options();

    /* GET TABLE DATASET AND PREPARE SCHEMAS */

    this.client.get('datasets/' + this.endpoint).then(x => {
      this.dataset = this.parse_response(x);
      let dataset = this.parse_response(x);
      let promises = [];

      /* Get columns schema */
      if (!this.schema) {
        if (this.columns) {
          let fields = (this.columns || []).concat((this.filtrables || []).filter((item) => this.columns.indexOf(item) < 0));
          this.schema = [];
          this.columns_schema = [];
          for (let field of fields) {
            if (field.indexOf('.') > 0) {
              let parts = field.split('.');
              let fieldDataset = dataset.find(o => o.field === parts[0]);
              let datasource = fieldDataset.datasource ? JSON.parse(fieldDataset.datasource) : null;
              if (datasource && datasource.table) {
                let fieldSchema = { field: `${parts[0]}.${parts[1]}`};
                this.schema.push(fieldSchema);
                if (this.columns.includes(fieldSchema.field)) this.columns_schema.push(fieldSchema);
                promises.push(this.client.get('datasets/' + datasource.table).then(x => {
                  let schema = this.parse_response(x).find(o => o.field === parts[1]) || {};
                  schema.field = fieldSchema.field;
                  this.schema[this.schema.findIndex(o => o.field === fieldSchema.field)] = schema;
                  if (this.columns.includes(fieldSchema.field)) this.columns_schema[this.columns_schema.findIndex(o => o.field === fieldSchema.field)] = schema;
                  this.dataset.push(schema);
                }));
              }
            } else {
              let fieldSchema = dataset.find(o => o.field === field) || Object.assign({}, this.config_metas.find(o => o.field === field));
              this.schema.push(fieldSchema);
              if (this.columns.includes(field)) this.columns_schema.push(fieldSchema);
            }
          }

        } else {
          this.columns_schema = this.schema = dataset;
        }
      }
      Promise.all(promises).finally(() => {
        this.schemaLoaded = true;
        this.schema_loaded();

        /* Parse sortables */
        for (let column of this.schema) {
          let field = column.field;
          if (field) {
            let fieldKebab = field.replace(/([a-z])([A-Z])/g, '$1-$2').replace(/\s+/g, '-').toLowerCase();
            if (this.sortables && this.sortables.includes(field)) {
              this.settings.sort_columns[field] = field;
            } else if (this.sortables && this.sortables.includes(fieldKebab)) {
              this.settings.sort_columns[field] = fieldKebab;
            } else if (!this.sortables) {
              this.settings.sort_columns[field] = field;
            }
          }
        }


        /* Get export schema */
        if (this.exportables && !this.export_schema) {
          this.export_schema = [];
          for (let field of this.exportables) {
            let export_field = Object.assign({}, dataset.find(o => o.field === field) || Object.assign({}, this.config_metas.find(o => o.field === field)));
            this.export_schema.push(export_field);
          }
        } else if (!this.export_schema) {
          this.export_schema = [];
          for (let field of dataset) {
            let export_field = Object.assign({}, field);
            this.export_schema.push(export_field);
          }
        }

        /* Get filters schema */
        if (this.filtrables && !this.filters) {
          this.filters = [];
          for (let field of this.filtrables) {
            let filter = Object.assign({}, dataset.find(o => o.field === field) || Object.assign({}, this.config_metas.find(o => o.field === field)));
            if (filter.control === 'combo') filter.datamultiple = true;
            if (filter.datatype === 'json') filter.control = 'text';
            if (filter.control === 'textarea' || filter.control === 'editor') filter.control = 'text';
            this.filters.push(filter);
          }
        } else if (!this.filters) {
          this.filters = [];
          for (let filter of dataset) {
            filter = Object.assign({}, filter);
            if (filter.control === 'combo') filter.datamultiple = true;
            if (filter.datatype === 'json') filter.control = 'text';
            if (filter.control === 'textarea' || filter.control === 'editor') filter.control = 'text';
            this.filters.push(filter);
          }
        }
        this.filters_loaded();

        //Schemas ready, now we can load records
        this.load();
      });
    }).catch(error => {
      console.error(error);
      this.isLoading = false;
      this.records = null;
      this.error = 'Errore nel caricamento dei dati';
    });
  }

  load() {
    this.isLoading = true;
    this.error = null;
    this.hide_options();

    /* Init exports model */
    if (!this.settings.exports) this.settings.exports = {};

    /* Init filters model */
    if (!this.settings.filters) this.settings.filters = {};
    for (let field of this.filters) {
      if (!this.settings.filters[field.field]) {
        let default_operator = null;
        if (field.datamultiple) default_operator = '~~';
        else if (field.control === 'number') default_operator = '=';
        else if (field.control === 'age') default_operator = '=';
        this.settings.filters[field.field] = { operator: default_operator, value: null, value_from: null, value_to: null, control: field.control };
      }
      if (field.control === 'age') {
        field.control = 'number';
        field.datatype = 'tinyint(3)';
      }
    }

    /* Build query */
    let query = this.query + (this.query.indexOf('?') >= 0 ? '&' : '?') + 'meta=1';
    if (this.depth) query += '&depth=' + this.depth;
    if (this.prefilter) query += '&filters=' + encodeURIComponent(this.prefilter);
    let filters = [];
    for (let field of Object.keys(this.settings.filters)) {
      let value = this.settings.filters[field].value || null;
      if (value && this.settings.filters[field].control === 'age') {
        value = parseInt(value, 10);
        let operator = this.settings.filters[field].operator || null;
        if (operator === '<=') filters.push(encodeURIComponent( field + '>=' + moment().subtract(parseInt(value, 10) + 1, 'years').format('YYYY-MM-DD') ));
        else if (operator === '>=') filters.push(encodeURIComponent( field + '<' + moment().subtract(value, 'years').format('YYYY-MM-DD') ));
        else filters.push(encodeURIComponent( field + '<' + moment().subtract(value, 'years').format('YYYY-MM-DD') + '&' + field + '>=' + moment().subtract(value + 1, 'years').format('YYYY-MM-DD') ));
      }
      else if (value && this.settings.filters[field].control === 'combo') {
        let operator = this.settings.filters[field].operator || '=';
        let filter = this.filters.find(x => x.field === field);
        if (filter && Array.isArray(filter.datasource)) {
          let item = filter.datasource.find(x => x.value === value);
          if (item && item.filter_operator) operator = item.filter_operator;
          if (item && item.filter_value) value = item.filter_value;
        }
        filters.push(encodeURIComponent(field + operator + value ));
      }
      else if (value) filters.push(encodeURIComponent(field + (this.settings.filters[field].operator ? this.settings.filters[field].operator : '=') + value ));
      if (this.settings.filters[field].value_from) filters.push(encodeURIComponent(field + '>=' + moment(this.settings.filters[field].value_from, 'YYYY-MM-DD').format('YYYY-MM-DD 00:00:00') ));
      if (this.settings.filters[field].value_to) filters.push(encodeURIComponent(field + '<=' + moment(this.settings.filters[field].value_to, 'YYYY-MM-DD').format('YYYY-MM-DD 23:59:59') ));
    }
    if (filters.length) {
      this.hasFilters = true;
      query += (this.prefilter ? '%26' : '&filters=') + filters.join('%26');
    }
    else this.hasFilters = false;

    /* BACKWARDS COMPATIBILITY before X-Total-Count */
    let count_query = this.endpoint + query + '&fields=id&depth=0&meta=0';

    /* Continue building query */
    if (this.settings.sort_stack.length) {
      let sorts = [];
      for (let field of this.settings.sort_stack)
        if (this.settings.sort[field] && this.settings.sort[field].order) sorts.push(encodeURIComponent((this.settings.sort[field].order == 'asc' ? '+' : '-') + field));
      query += '&sort=' + sorts.join(',');
    }
    /* Setup reference query */
    // Used for ka-resource prev|next record navigation
    this.reference_query = this.endpoint + query;

    /* Continue building query */
    if (this.settings.limit) {
      query += '&limit=' + this.settings.limit;
    }
    if (this.settings.offset) {
      query += '&offset=' + this.settings.offset;
    }
    
    /* Setup export query */
    this.export_query = this.endpoint + query;

    query = this.parse_query(query);

    /* Get records */
    this.client.get(this.endpoint + query).then(x => {
      this.records = this.parse_response(x);
      this.isLoading = false;
      this.records_loaded();
      if (window[`ka-table-resize-handler-${this.id}`]) setTimeout(() => { window[`ka-table-resize-handler-${this.id}`](); }, 0);
      new Promise((resolve) => {
        if (x.headers && x.headers.headers && x.headers.headers['x-total-count']) return resolve(x.headers.headers['x-total-count'].value);
        this.client.get(count_query).then(x2 => { resolve(this.parse_response(x2).length); });
      }).then((total) => {
        this.total = total;
        this.position = (this.records.length > 0 ? parseInt(this.settings.offset)+1 : 0) + '-' + (parseInt(this.settings.offset)+parseInt(this.settings.limit) > this.total ? this.total : this.records.length == parseInt(this.settings.limit) ? parseInt(this.settings.offset) + parseInt(this.settings.limit) : parseInt(this.settings.offset) + this.records.length);
        for (let i = 1; i <= Math.ceil(this.total / parseInt(this.settings.limit)); i++) {
          if (i <= 1) this.config_options.pages.datasource = [{text: String(i), value: String(i)}];
          else this.config_options.pages.datasource.push({text: String(i), value: String(i)});
        }
      });
    }).catch(error => {
      console.error(error);
      this.isLoading = false;
      this.records = null;
      this.error = 'Errore nel caricamento dei dati';
    });
    /* Store table settings on local storage */
    localStorage.setItem(this.id, JSON.stringify(this.settings));
  }

  /* Formats record output depending on kamaji's field type */
  format(record, field) {
    let value = null;
    let column = this.dataset.find(x => x.field === field);

    if (!column && !['date_created', 'date_modified'].includes(field)) return record[field];

    if (field.indexOf('.') > 0) {
      let parts = field.split('.');
      value = record[parts[0]] && record[parts[0]][parts[1]] ? record[parts[0]][parts[1]] : null;
    } else {
      value = record[field];
    }

    if (!value) return null;

    if (field === 'date_created' || field === 'date_modified') {
      return moment.utc(value, 'YYYY-MM-DD').local().format('DD/MM/YYYY');
    } else if (column.control === 'list' && column.datasource) {
      if (typeof column.datasource === 'string' && (column.datasource.indexOf('{') === 0 || column.datasource.indexOf('[') === 0)) {
        column.datasource = JSON.parse(column.datasource);
      }
      let view = column.datasource.model.view || null;
      let keys = !view ? null : view.match(/\{[a-zA-Z0-9_\.]*?\}/g);
      let values = JSON.parse(value);
      let join = column.datasource.model.join || ', ';
      if (view && keys) {
        values.forEach((v, i) => {
          let nw = view;
          for (let key of keys) nw = nw.replace(key, v[key.replace(/\{|\}/g, '')] || '');
          values[i] = nw;
        });
      } else {
        values.forEach((v, i) => {
          values[i] = Object.values(v).join(' ');
        });
      }
      return values.join(join);
    } else if (column.datamultiple && column.datasource) {
      let values = [];
      for (let v of value) {
        let keys = column.datatext.match(/\{[a-zA-Z0-9_\.]*?\}/g);
        if (keys) {
          let datatext = column.datatext;
          for (let key of keys) {
            datatext = datatext.replace(key, v[key.replace(/\{|\}/g, '')] || '');
          }
          values.push(datatext);
        }
        else values.push(v[column.datatext || 'text']);
      }
      return values.join(', ');
    } else if (!column.datamultiple && column.datasource && column.control !== 'file') {
      let keys = column.datatext.match(/\{[a-zA-Z0-9_\.]*?\}/g);
      if (typeof column.datasource === 'string' && (column.datasource.indexOf('{') === 0 || column.datasource.indexOf('[') === 0)) {
        column.datasource = JSON.parse(column.datasource);
      }
      if (keys) {
        let datatext = column.datatext;
        for (let key of keys) {
          datatext = datatext.replace(key, value[key.replace(/\{|\}/g, '')] || '');
        }
        return datatext;
      } else if (typeof value === 'string' && Array.isArray(column.datasource)) {
        let v = column.datasource.filter(x => { return String(x[column.datavalue]) === String(value); });
        if (v && v.length) return v[0][column.datatext];
        return value;
      }
      return value[column.datatext || 'text'];
    } else if (column.datatype === 'json') {
      return JSON.stringify(value);
    } else if (column.control === 'age') {
      return moment().diff(moment(value, 'YYYY-MM-DD'), 'years');
    } else if (column.datatype === 'date') {
      return (column.utc ? moment.utc(value, 'YYYY-MM-DD').local() : moment(value, 'YYYY-MM-DD')).format('DD/MM/YYYY');
    } else if (column.datatype === 'datetime') {
      return (column.utc ? moment.utc(value, 'YYYY-MM-DD HH:mm:ss').local() : moment(value, 'YYYY-MM-DD HH:mm:ss')).format('DD/MM/YYYY HH:mm:ss');
    }
    return value;
  }

  set_sort(field) {
    let sort = this.settings.sort_stack.indexOf(field) > -1 ? this.settings.sort[field] || null : null;
    if (sort) {
      sort.order = (sort.order === 'asc' ? 'desc' : (sort.order === 'desc' ? null : 'asc'));
      if (!sort.order) {
        this.settings.sort[field] = {};
        this.settings.sort_stack.splice(this.settings.sort_stack.indexOf(field), 1);
      }
    }
    else {
      this.settings.sort_stack.push(field);
      this.settings.sort[field] = { order: 'asc' };
    }
    for (let f of this.settings.sort_stack) {
      this.settings.sort[f].class = 'sort-' + this.settings.sort[f].order + ' sort-' + (this.settings.sort_stack.indexOf(f) + 1);
    }
    this.load();
  }

  set_page (page) {
    let pages = Math.ceil(this.total / parseInt(this.settings.limit));
    if (page == 'first')
      this.settings.offset = 0;
    else if (page == 'prev')
      this.settings.offset = this.settings.offset - parseInt(this.settings.limit) > 0 ? this.settings.offset - parseInt(this.settings.limit) : 0;
    else if (page == 'next')
      this.settings.offset = !this.records.length ? this.settings.offset : this.settings.offset + parseInt(this.settings.limit) < this.total ? this.settings.offset + parseInt(this.settings.limit) : parseInt(this.settings.limit) * (pages -1);
    else if (page == 'last')
      this.settings.offset = !this.records.length ? this.settings.offset : parseInt(this.settings.limit) * (pages -1);
    else if (typeof page == 'number' || (/^[0-9]*?$/g).test(page))
      this.settings.offset = parseInt(this.settings.limit) * (parseInt(page) - 1);
    else return;
    this.load();
  }

  set_limit(limit) {
    this.settings.limit = limit;
    this.load();
  }

  show_options ($event) {
    if (this.optionsOpen) return this.hide_options();
    // Add backdrop
    let backdrop = document.createElement('div');
    backdrop.classList.add('ka-table-backdrop');
    backdrop.setAttribute('id', 'ka-table-backdrop-' + this.id);
    backdrop.addEventListener('click', event => { if (event.target == backdrop) this.hide_options(); });
    document.getElementsByTagName('body')[0].insertBefore(backdrop, document.getElementsByTagName('body')[0].children[0]);
    // Move options box to backdrop and open
    let drawer = document.querySelectorAll('#'+this.id+' .ka-table-options')[0];
    if ($event && $event.target) {
      let bounds = $event.target.getBoundingClientRect();
      drawer.style.top = bounds.top+'px';
      drawer.style.left = bounds.left+(bounds.width/2)+'px';
    }
    backdrop.appendChild(drawer);
    drawer.classList.remove('hidden');
    setTimeout(()=>{ // Timeout used for css transition
      drawer.classList.add('open');
      this.optionsOpen = true;
    }, 100);
  }
  hide_options () {
    if (!this.optionsOpen) return;
    // Move options box back to component
    let drawer = document.querySelectorAll('#ka-table-backdrop-'+this.id+' .ka-table-options')[0];
    drawer.classList.remove('open');
    setTimeout(()=>{
      drawer.classList.add('hidden');
      this.element.appendChild(drawer);
      // Remove backdrop
      document.getElementsByTagName('body')[0].removeChild(document.getElementById('ka-table-backdrop-' + this.id));
      // Close box
      this.optionsOpen = false;
    }, 500);
  }

  show_filters () {
    if (this.filtersOpen) return this.hide_filters();
    // Add backdrop
    let backdrop = document.createElement('div');
    backdrop.classList.add('ka-table-backdrop');
    backdrop.setAttribute('id', 'ka-table-backdrop-' + this.id);
    backdrop.addEventListener('click', event => { if (event.target == backdrop) this.hide_filters(); });
    document.getElementsByTagName('body')[0].insertBefore(backdrop, document.getElementsByTagName('body')[0].children[0]);
    // Move drawer to backdrop and open
    let drawer = document.querySelectorAll('#'+this.id+' .ka-table-drawer.filters')[0];
    backdrop.appendChild(drawer);
    drawer.classList.remove('hidden');
    setTimeout(()=>{ // Timeout used for css transition
      drawer.classList.add('open');
      this.filtersOpen = true;
    }, 100);
  }
  hide_filters () {
    // Move drawer back to component
    let drawer = document.querySelectorAll('#ka-table-backdrop-'+this.id+' .ka-table-drawer.filters')[0];
    if (!drawer) return; // Drawer may not exist if applied filters from ka-tableFilters component
    drawer.classList.remove('open');
    setTimeout(()=>{
      drawer.classList.add('hidden');
      this.element.appendChild(drawer);
      // Remove backdrop
      document.getElementsByTagName('body')[0].removeChild(document.getElementById('ka-table-backdrop-' + this.id));
      // Close drawer
      this.filtersOpen = false;
    }, 500);
  }
  apply_filters () {
    this.hide_filters();
    this.settings.offset = 0;
    this.load();
  }
  reset_filters () {
    if (this.filtersOpen) this.hide_filters();
    this.settings.offset = 0;
    for (let field of this.filters) {
      if (this.settings.filters[field.field]) {
        let default_operator = null;
        if (field.datamultiple) default_operator = '~~';
        else if (field.control === 'number') default_operator = '=';
        else if (field.control === 'age') default_operator = '=';
        this.settings.filters[field.field] = { operator: default_operator, value: null, value_from: null, value_to: null, control: field.control };
      }
      if (field.control === 'age') {
        field.control = 'number';
        field.datatype = 'tinyint(3)';
      }
    }
    this.load();
  }

  show_export () {
    if (this.exportOpen) return this.hide_export();
    // Add backdrop
    let backdrop = document.createElement('div');
    backdrop.classList.add('ka-table-backdrop');
    backdrop.setAttribute('id', 'ka-table-backdrop-' + this.id);
    backdrop.addEventListener('click', event => { if (event.target == backdrop) this.hide_export(); });
    document.getElementsByTagName('body')[0].insertBefore(backdrop, document.getElementsByTagName('body')[0].children[0]);
    // Move drawer to backdrop and open
    let drawer = document.querySelectorAll('#'+this.id+' .ka-table-drawer.export')[0];
    backdrop.appendChild(drawer);
    drawer.classList.remove('hidden');
    setTimeout(()=>{ // Timeout used for css transition
      drawer.classList.add('open');
      this.exportOpen = true;
    }, 100);
  }
  hide_export () {
    // Move drawer back to component
    let drawer = document.querySelectorAll('#ka-table-backdrop-'+this.id+' .ka-table-drawer.export')[0];
    drawer.classList.remove('open');
    setTimeout(()=>{
      drawer.classList.add('hidden');
      this.element.appendChild(drawer);
      // Remove backdrop
      document.getElementsByTagName('body')[0].removeChild(document.getElementById('ka-table-backdrop-' + this.id));
      // Close drawer
      this.exportOpen = false;
    }, 500);
  }
  select_export() {
    for (let field of this.export_schema) {
      this.settings.exports[field.field] = true;
    }
  }
  unselect_export() {
    for (let field of this.export_schema) {
      this.settings.exports[field.field] = false;
    }
  }
  apply_export($event, exportPages = null) {
    if ($event && $event.model) $event.model.busy = true;
    let fields = Object.keys(this.settings.exports).reduce((acc, k) => { if (this.settings.exports[k] === true) acc.push(k); return acc; }, []).join(',');
    let query = this.export_query;
    if (exportPages === 'ALL') {
      query = query.replace(/&limit=\d*/, '');
      query = query.replace(/&offset=\d*/, '');
    }
    this.client.get(query + '&depth=1&meta=0&fields=' + fields).then(x => {
      let records = this.parse_response(x);
      if (!records.length) {
        this.toast.show('<strong>Nessun risulta alcun dato da esportare!</strong>', 'error');
        return;
      }
      let csv = [Object.keys(records[0]).join(';')];
      for (let record of records) {
        let values = [];
        for (let k of Object.keys(record)) {
          let value = this.dataset.find(f => { return f.field === k; }) ? this.format(record, k) : record[k];
          if (typeof value === 'string') {
            value = value.replace('"', '""'); // CSV escape double quotes
            value = `"${value}"`; // CSV double quote value delimiter
          }
          values.push(value);
        }
        csv.push(values.join(';'));
      }
      csv = csv.join('\n');
      let blob = new Blob([csv], {type: 'text/csv'});
      let anchor = document.createElement('a');
      anchor.download = moment().format('YYYY-MM-DD_HH-mm') + '_' + this.endpoint.replace(/[^a-zA-Z0-9]/g, '-') + '.csv';
      anchor.href = window.URL.createObjectURL(blob);
      document.body.appendChild(anchor);
      anchor.click();
      document.body.removeChild(anchor);
      this.hide_export();
    }).finally(() => {
      if ($event && $event.model) $event.model.busy = false;
    });
  }

  add () {
    if (this.routes.edit === false) return;
    if (this.routes.dialog) {
      let childRoute = this.router.routes.filter(x => { return x.name == (this.routes.add || this.router.currentInstruction.config.name) })[0];
      this.dialog.open({
        title: childRoute.title,
        viewModel: childRoute.moduleId,
        viewModelParams: { id: 'new', data: this.routes.data || {} }
      }, true).whenClosed(response => {
        if (!response.wasCancelled) {
          this.load();
        }
      });
    }
    else this.router.navigateToRoute(this.routes.add || this.router.currentInstruction.config.name, {id: 'new'});
  }
  edit (record) {
    let listRefQuery = `${this.reference_query}&fields=id&limit=1&offset=${parseInt(this.settings.offset || 0, 10) + this.records.indexOf(record)}`;
    if (this.actions.indexOf('edit') < 0) return;
    if (this.routes.edit === false) return;
    if (this.routes.dialog) {
      let childRoute = this.router.routes.filter(x => { return x.name == (this.routes.edit || this.router.currentInstruction.config.name) })[0];
      this.dialog.open({
        title: childRoute.title,
        viewModel: childRoute.moduleId,
        viewModelParams: { id: record.id || record.uuid, listRefQuery: btoa(listRefQuery) }
      }, true).whenClosed(response => {
        if (!response.wasCancelled) {
          this.load();
        }
      });
    }
    else this.router.navigateToRoute(this.routes.edit || this.router.currentInstruction.config.name, {id: record.id || record.uuid, listRefQuery: btoa(listRefQuery)});
  }
  copy (record, deep = false) {
    if (this.routes.edit === false) return;
    if (deep) {
      let fields = this.dataset.filter(x => { return x.unique /*|| (x.required && x.control == 'password')*/ });
      let duplicate = (data) => {
        let d = this.params && this.params.deepcopy ? JSON.parse(JSON.stringify(this.params.deepcopy)) : { depth: 1 };
        d.data = data;
        this.client.post(`duplicate/${record.id || record.uuid}`, d).then(x => {
          this.toast.show(`<strong>Record duplicato</strong>`, 'success');
          this.load();
        }).catch(x => {
          let error = JSON.parse(x.response).errors[0].msg;
          this.toast.show(`<strong>${error}</strong>`, 'error');
        });
      }
      if (fields.length) {
        let template = '', config = {}, model = {};
        for (let field of fields) {
          config[field.field] = field;
          model[field.field] = null;
          template += `<ka-input config.bind="params.viewStrategyParams.config.${field.field}" value.bind="params.viewStrategyParams.model.${field.field}"></ka-input>`;
        }
        let viewStrategy = new InlineViewStrategy(`<template>${template}</template>`);
        let viewStrategyParams = { config: config, model: model };
        this.dialog.open({
          type: 'confirm',
          title: 'Duplica record',
          viewStrategy: viewStrategy,
          viewStrategyParams: viewStrategyParams,
        }, true).whenClosed(response => {
          console.log(viewStrategyParams.model);
          if (!response.wasCancelled) {
            duplicate(viewStrategyParams.model);
          }
        });
      }
      else duplicate();
    }
    else {
      this.client.get(`${this.endpoint}/${record.id || record.uuid}`, {meta: 0, depth: 0}).then(x => {
        let data = x.response;
        delete data.id;
        delete data.uuid;
        //delete data.endpoint;
        //delete data.user_created;
        //delete data.user_modified;
        //delete data.date_created;
        //delete data.date_modified;
        if (this.routes.dialog) {
          let childRoute = this.router.routes.filter(x => { return x.name == (this.routes.edit || this.router.currentInstruction.config.name) })[0];
          this.dialog.open({
            title: childRoute.title,
            viewModel: childRoute.moduleId,
            viewModelParams: { id: 'new', data: data }
          }, true).whenClosed(response => {
            if (!response.wasCancelled) {
              this.load();
            }
          });
        } else this.router.navigateToRoute(this.routes.edit || this.router.currentInstruction.config.name, {id: 'new', data: data});
      });
    }
  }
  delete (record) {
    if (this.routes.edit === false) return;
    this.dialog.open({
      type: 'confirm',
      class: 'warning',
      title: 'Elimina',
      body: 'Eliminare definitivamente il record?'
    }, true).whenClosed(response => {
      if (!response.wasCancelled) {
        this.client.delete(`${this.endpoint}/${record.id || record.uuid}`).then(x => {
          this.toast.show(`<strong>Record eliminato</strong>`, 'success');
          this.load();
        }).catch(x => {
          let error = JSON.parse(x.response).errors[0].msg;
          this.toast.show(`<strong>${error}</strong>`, 'error');
        });
      }
    });
  }
  parse_response (msg) {
    if (msg.responseType === 'json' && typeof msg.response === 'string') msg.response = JSON.parse(msg.response);
    return msg.response;
  }
}
