import {customElement, containerless, inject, bindable, bindingMode} from 'aurelia-framework';

@customElement('ka-table-filters')
@containerless()
@inject(Element)
export class KaTableFilters {
  @bindable() schema = null;
  @bindable({defaultBindingMode: bindingMode.twoWay}) filters = {};
  @bindable() apply = () => {};
  @bindable() reset = () => {};
  _schema = null;

  constructor(element) {
    this.element = element;
  }

  schemaChanged(schema) {
    schema = JSON.parse(JSON.stringify(schema));
    // Validate schema
    if (!schema) {
      konsole.warn('ka-table-filters: missing schema!', schema);
      return;
    }
    if (typeof schema === 'string') {
      try {
        schema = JSON.parse(schema);
      } catch (error) {
        konsole.error('ka-table-filters: invalid schema provided!', schema);
        return;
      }
    }
    if (!Array.isArray(schema) || !schema.length) {
      konsole.error('ka-table-filters: schema must be an array of objects!', schema);
      return;
    }

    // Override schema parameters
    for (let item of schema) {
      item.placeholder = `Filtra per ${item.label.toLowerCase()}`;
      item.label = null;
      item.required = false;
    }

    konsole.debug('ka-table-filters: schema changed!', schema);
    this._schema = schema;
  }
}
